﻿using Infragistics.Windows.DockManager;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;

namespace XamDockManager_sample
{
    public static class ContentPaneEvents
    {
        #region RaiseDockEvents

        public static readonly DependencyProperty RaiseDockEventsProperty =
            DependencyProperty.RegisterAttached("RaiseDockEvents", typeof(bool), typeof(ContentPaneEvents),
                new FrameworkPropertyMetadata(false, new PropertyChangedCallback(OnRaiseDockEventsChanged)));

        private static void OnRaiseDockEventsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ContentPane pane = d as ContentPane;

            if (null != pane)
            {
                if (true.Equals(e.NewValue))
                    pane.SetBinding(ContentPaneEvents.PaneLocationValueProperty, Infragistics.Windows.Utilities.CreateBindingObject(ContentPane.PaneLocationProperty, BindingMode.OneWay, pane));
                else
                    pane.ClearValue(ContentPaneEvents.PaneLocationValueProperty);
            }
        }

        [AttachedPropertyBrowsableForType(typeof(ContentPane))]
        public static bool GetRaiseDockEvents(DependencyObject d)
        {
            return (bool)d.GetValue(RaiseDockEventsProperty);
        }

        public static void SetRaiseDockEvents(DependencyObject d, bool value)
        {
            d.SetValue(RaiseDockEventsProperty, value);
        }

        #endregion // RaiseDockEvents

        #region PaneLocationValue

        private static readonly DependencyProperty PaneLocationValueProperty =
            DependencyProperty.RegisterAttached("PaneLocationValue", typeof(PaneLocation?), typeof(ContentPaneEvents),
                new FrameworkPropertyMetadata(null, new PropertyChangedCallback(OnPaneLocationValueChanged)));

        private static void OnPaneLocationValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ContentPane pane = d as ContentPane;

            // the value is null when being initialized so don't raise the event
            if (e.OldValue != null && null != pane)
            {
                RoutedEventArgs args = null;

                if (pane.PaneLocation == PaneLocation.Floating || pane.PaneLocation == PaneLocation.FloatingOnly)
                {
                    args = new RoutedEventArgs(UndockedEvent);
                    pane.RaiseEvent(args);
                }
                else if (pane.PaneLocation != PaneLocation.Unknown && pane.PaneLocation != PaneLocation.Unpinned)
                {
                    args = new RoutedEventArgs(DockedEvent);
                    pane.RaiseEvent(args);
                }
            }
        }

        #endregion // PaneLocationValue

        #region Docked

        public static readonly RoutedEvent DockedEvent = EventManager.RegisterRoutedEvent("Docked",
            RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(ContentPaneEvents));

        public static void AddDockedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.AddHandler(DockedEvent, handler);
        }

        public static void RemoveDockedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.RemoveHandler(DockedEvent, handler);
        }
        #endregion // Docked

        #region Undocked

        public static readonly RoutedEvent UndockedEvent = EventManager.RegisterRoutedEvent("Undocked",
            RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(ContentPaneEvents));

        public static void AddUndockedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.AddHandler(UndockedEvent, handler);
        }

        public static void RemoveUndockedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.RemoveHandler(UndockedEvent, handler);
        }
        #endregion // Undocked
    }
}
